/* speaker-toggle: a program to toggle the GP2X speakers on/off,
 * leaving the headphones on.
 *   -- Gordon Ingram (ggi)
 *
 * This is a stripped-down version of the mixer program `rexima',
 * version 1.4, which is:
 *    Copyright (C) 1996-2003 Russell Marks.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <signal.h>

char dev_labels[SOUND_MIXER_NRDEVICES][80]=SOUND_DEVICE_LABELS;
char dev_names [SOUND_MIXER_NRDEVICES][80]=SOUND_DEVICE_NAMES;
int dev_line[SOUND_MIXER_NRDEVICES];

char *mixerdev="/dev/mixer";

void die(char *str)
{
  fprintf(stderr,"speaker-toggle: couldn't %s.\n",str);
  exit(1);
}

void init(int *mixfd,int *existmask,int *canrecmask,int *isrecmask,
          int *stereomask)
{
  if(((*mixfd)=open(mixerdev,O_RDWR))<0) die("open mixer device");
  if(ioctl(*mixfd,SOUND_MIXER_READ_DEVMASK,existmask)==-1) die("ioctl");
  if(ioctl(*mixfd,SOUND_MIXER_READ_RECMASK,canrecmask)==-1) die("ioctl");
  if(ioctl(*mixfd,SOUND_MIXER_READ_RECSRC,isrecmask)==-1) die("ioctl");
  /* this looks like a `recent' addition [in 1996 maybe ;-)] so be lenient */
  if(ioctl(*mixfd,SOUND_MIXER_READ_STEREODEVS,stereomask)==-1) *stereomask=0;
}

void uninit(int mixfd)
{
  close(mixfd);
}

int mixer_getlevel_stereo(int mixfd,int dev)
{
  int level=0;
  
  ioctl(mixfd,MIXER_READ(dev),&level);
  return(level);
}

void mixer_setlevel_stereo(int mixfd,int dev,int left,int right)
{
  left+=256*right;
  ioctl(mixfd,MIXER_WRITE(dev),&left);
}

int main(int argc, char *argv[])
{
  int mixfd;
  int existmask,canrecmask,isrecmask,stereomask;
  int f;
  int found;
  
  init(&mixfd,&existmask,&canrecmask,&isrecmask,&stereomask);

  found = 0;
  for(f = 0; f < SOUND_MIXER_NRDEVICES; f++) {
    if(existmask & (1 << f) && strcmp("vol", dev_names[f])==0) {
	found = 1;
	break;
      }
  }

   // Go daemon (so we can loop in background for mute mode)
   signal(SIGTTOU, SIG_IGN);
   signal(SIGTTIN, SIG_IGN);
   signal(SIGTSTP, SIG_IGN);
   fclose(stdout);fclose(stderr);

   int pid;
   pid = fork();
   if (pid < 0) {
     exit(1);
   }
   if (pid != 0) {
     exit(0); //leave the father immediatly
   }

   setsid(); // new session
   signal(SIGHUP, SIG_IGN);
   pid = fork();
   if (pid < 0) {
     exit(1);
   }
   if (pid != 0) {
     exit(0);
   }

  if (mixer_getlevel_stereo(mixfd, f) > 0) {
    // Loop on mute
    while(1) {
      mixer_setlevel_stereo(mixfd, f, 0, 0);
      sleep(1);
    }
  } else {
    // Restore normal level
    mixer_setlevel_stereo(mixfd, f, 67, 67);
  }

  uninit(mixfd);
  return 0;
}
